# -*- coding: utf-8 -*-

'''ahl_dataset.py: Contains the non-Euclidean space datasets of AHL (Akutten
og Hjerte-lunge-senteret) at St. Olavs Hospital.'''

import dataset_definition as datadef


def ahl_graph(n):
    '''Returns n connected floors of AHL.'''
    # n must be positive
    if 0 > n:
        return datadef.Hospital_graph()

    # Create the first floor of AHL
    graph = datadef.Hospital_graph()
    graph, conn = create_ahl_floor_1(graph, 1)

    # If n=1 then we only need to return the first floor
    if n == 1:
        graph.reset_edge_weights()
        return graph

    # Create n-1 floors and connect them together
    for i in xrange(2, n+1):
        graph_2 = datadef.Hospital_graph()
        graph_2, conn_2 = create_ahl_floor_1(graph_2, i)

        # Connect the two graphs via the connectors, e.g. stairs.
        graph = connect_graphs(graph, graph_2, conn, conn_2)
        if not graph:
            return False
        conn = conn_2

    # Initialize edge weights
    graph.reset_edge_weights()

    return graph


def connect_graphs(graph_1, graph_2, conn_1, conn_2):
    '''Connects two graphs by adding nodes and edges from graph_2 to graph_1
    and connecting them via the two connector arguments.'''
    # The amount of connector nodes in each graph must match
    if len(conn_1) != len(conn_2):
        return False

    # Add nodes and edges from graph_2 to graph_1
    graph_1.add_nodes_from(graph_2.nodes())
    graph_1.add_edges_from(graph_2.edges())

    # Connect connectors together by adding an edge between the nodes
    i = 0
    for node_1 in conn_1:
        graph_1.add_edge(node_1, conn_2[i])
        i += 1

    return graph_1


def create_ahl_floor_1(graph, floor_id=None):
    '''Generates a graph of the first floor of the AHL building
    (Akutten og Hjerte-lunge-senteret) at St. Olavs Hospital'''
    if not floor_id:
        floor_id = 1
    #
    # Create nodes
    #
    ahl01s001 = datadef.Node('AHL.0%s.S001' % floor_id)  # stairs
    ahl01s002 = datadef.Node('AHL.0%s.S002' % floor_id)
    ahl01s003 = datadef.Node('AHL.0%s.S003' % floor_id)
    ahl01s004 = datadef.Node('AHL.0%s.S004' % floor_id)
    ahl01s005 = datadef.Node('AHL.0%s.S005' % floor_id)
    ahl01s006 = datadef.Node('AHL.0%s.S006' % floor_id)
    ahl01s007 = datadef.Node('AHL.0%s.S007' % floor_id)

    ahl01e001 = datadef.Node('AHL.0%s.E001' % floor_id)  # elevators
    ahl01e002 = datadef.Node('AHL.0%s.E002' % floor_id)
    ahl01e003 = datadef.Node('AHL.0%s.E003' % floor_id)
    ahl01e004 = datadef.Node('AHL.0%s.E004' % floor_id)
    ahl01e005 = datadef.Node('AHL.0%s.E005' % floor_id)
    ahl01e006 = datadef.Node('AHL.0%s.E006' % floor_id)
    ahl01e007 = datadef.Node('AHL.0%s.E007' % floor_id)
    ahl01e008 = datadef.Node('AHL.0%s.E008' % floor_id)

    ahl01c001 = datadef.Node('AHL.0%s.C001' % floor_id)  # corridors
    ahl01c002 = datadef.Node('AHL.0%s.C002' % floor_id)
    ahl01c003 = datadef.Node('AHL.0%s.C003' % floor_id)
    ahl01c004 = datadef.Node('AHL.0%s.C004' % floor_id)
    ahl01c005 = datadef.Node('AHL.0%s.C005' % floor_id)
    ahl01c006 = datadef.Node('AHL.0%s.C006' % floor_id)
    ahl01c007 = datadef.Node('AHL.0%s.C007' % floor_id)
    ahl01c008 = datadef.Node('AHL.0%s.C008' % floor_id)
    ahl01c009 = datadef.Node('AHL.0%s.C009' % floor_id)
    ahl01c010 = datadef.Node('AHL.0%s.C010' % floor_id)
    ahl01c011 = datadef.Node('AHL.0%s.C011' % floor_id)
    ahl01c012 = datadef.Node('AHL.0%s.C012' % floor_id)
    ahl01c013 = datadef.Node('AHL.0%s.C013' % floor_id)
    ahl01c014 = datadef.Node('AHL.0%s.C014' % floor_id)
    ahl01c015 = datadef.Node('AHL.0%s.C015' % floor_id)
    ahl01c016 = datadef.Node('AHL.0%s.C016' % floor_id)
    ahl01c017 = datadef.Node('AHL.0%s.C017' % floor_id)
    ahl01c018 = datadef.Node('AHL.0%s.C018' % floor_id)
    ahl01c019 = datadef.Node('AHL.0%s.C019' % floor_id)
    ahl01c020 = datadef.Node('AHL.0%s.C020' % floor_id)
    ahl01c021 = datadef.Node('AHL.0%s.C021' % floor_id)
    ahl01c022 = datadef.Node('AHL.0%s.C022' % floor_id)
    ahl01c023 = datadef.Node('AHL.0%s.C023' % floor_id)
    ahl01c024 = datadef.Node('AHL.0%s.C024' % floor_id)
    ahl01c025 = datadef.Node('AHL.0%s.C025' % floor_id)
    ahl01c026 = datadef.Node('AHL.0%s.C026' % floor_id)
    ahl01c027 = datadef.Node('AHL.0%s.C027' % floor_id)
    ahl01c028 = datadef.Node('AHL.0%s.C028' % floor_id)
    ahl01c029 = datadef.Node('AHL.0%s.C029' % floor_id)
    ahl01c030 = datadef.Node('AHL.0%s.C030' % floor_id)
    ahl01c031 = datadef.Node('AHL.0%s.C031' % floor_id)
    ahl01c032 = datadef.Node('AHL.0%s.C032' % floor_id)
    ahl01c033 = datadef.Node('AHL.0%s.C033' % floor_id)
    ahl01c034 = datadef.Node('AHL.0%s.C034' % floor_id)
    ahl01c035 = datadef.Node('AHL.0%s.C035' % floor_id)
    ahl01c036 = datadef.Node('AHL.0%s.C036' % floor_id)
    ahl01c037 = datadef.Node('AHL.0%s.C037' % floor_id)

    ahl01r001 = datadef.Node('AHL.0%s.R001' % floor_id)  # rooms
    ahl01r002 = datadef.Node('AHL.0%s.R002' % floor_id)
    ahl01r003 = datadef.Node('AHL.0%s.R003' % floor_id)
    ahl01r004 = datadef.Node('AHL.0%s.R004' % floor_id)
    ahl01r005 = datadef.Node('AHL.0%s.R005' % floor_id)
    ahl01r006 = datadef.Node('AHL.0%s.R006' % floor_id)
    ahl01r007 = datadef.Node('AHL.0%s.R007' % floor_id)
    ahl01r008 = datadef.Node('AHL.0%s.R008' % floor_id)
    ahl01r009 = datadef.Node('AHL.0%s.R009' % floor_id)
    ahl01r010 = datadef.Node('AHL.0%s.R010' % floor_id)
    ahl01r011 = datadef.Node('AHL.0%s.R011' % floor_id)
    ahl01r012 = datadef.Node('AHL.0%s.R012' % floor_id)
    ahl01r013 = datadef.Node('AHL.0%s.R013' % floor_id)
    ahl01r014 = datadef.Node('AHL.0%s.R014' % floor_id)
    ahl01r015 = datadef.Node('AHL.0%s.R015' % floor_id)
    ahl01r016 = datadef.Node('AHL.0%s.R016' % floor_id)
    ahl01r017 = datadef.Node('AHL.0%s.R017' % floor_id)
    ahl01r018 = datadef.Node('AHL.0%s.R018' % floor_id)
    ahl01r019 = datadef.Node('AHL.0%s.R019' % floor_id)
    ahl01r020 = datadef.Node('AHL.0%s.R020' % floor_id)
    ahl01r021 = datadef.Node('AHL.0%s.R021' % floor_id)
    ahl01r022 = datadef.Node('AHL.0%s.R022' % floor_id)
    ahl01r023 = datadef.Node('AHL.0%s.R023' % floor_id)
    ahl01r024 = datadef.Node('AHL.0%s.R024' % floor_id)
    ahl01r025 = datadef.Node('AHL.0%s.R025' % floor_id)
    ahl01r026 = datadef.Node('AHL.0%s.R026' % floor_id)
    ahl01r027 = datadef.Node('AHL.0%s.R027' % floor_id)
    ahl01r028 = datadef.Node('AHL.0%s.R028' % floor_id)
    ahl01r029 = datadef.Node('AHL.0%s.R029' % floor_id)
    ahl01r030 = datadef.Node('AHL.0%s.R030' % floor_id)
    ahl01r031 = datadef.Node('AHL.0%s.R031' % floor_id)
    ahl01r032 = datadef.Node('AHL.0%s.R032' % floor_id)
    ahl01r033 = datadef.Node('AHL.0%s.R033' % floor_id)
    ahl01r034 = datadef.Node('AHL.0%s.R034' % floor_id)
    ahl01r035 = datadef.Node('AHL.0%s.R035' % floor_id)
    ahl01r036 = datadef.Node('AHL.0%s.R036' % floor_id)
    ahl01r037 = datadef.Node('AHL.0%s.R037' % floor_id)
    ahl01r038 = datadef.Node('AHL.0%s.R038' % floor_id)
    ahl01r039 = datadef.Node('AHL.0%s.R039' % floor_id)
    ahl01r040 = datadef.Node('AHL.0%s.R040' % floor_id)
    ahl01r041 = datadef.Node('AHL.0%s.R041' % floor_id)
    ahl01r042 = datadef.Node('AHL.0%s.R042' % floor_id)
    ahl01r043 = datadef.Node('AHL.0%s.R043' % floor_id)
    ahl01r044 = datadef.Node('AHL.0%s.R044' % floor_id)
    ahl01r045 = datadef.Node('AHL.0%s.R045' % floor_id)
    ahl01r046 = datadef.Node('AHL.0%s.R046' % floor_id)
    ahl01r047 = datadef.Node('AHL.0%s.R047' % floor_id)
    ahl01r048 = datadef.Node('AHL.0%s.R048' % floor_id)
    ahl01r049 = datadef.Node('AHL.0%s.R049' % floor_id)
    ahl01r050 = datadef.Node('AHL.0%s.R050' % floor_id)
    ahl01r051 = datadef.Node('AHL.0%s.R051' % floor_id)
    ahl01r052 = datadef.Node('AHL.0%s.R052' % floor_id)
    ahl01r053 = datadef.Node('AHL.0%s.R053' % floor_id)
    ahl01r054 = datadef.Node('AHL.0%s.R054' % floor_id)
    ahl01r055 = datadef.Node('AHL.0%s.R055' % floor_id)
    ahl01r056 = datadef.Node('AHL.0%s.R056' % floor_id)
    ahl01r057 = datadef.Node('AHL.0%s.R057' % floor_id)
    ahl01r058 = datadef.Node('AHL.0%s.R058' % floor_id)
    ahl01r059 = datadef.Node('AHL.0%s.R059' % floor_id)
    ahl01r060 = datadef.Node('AHL.0%s.R060' % floor_id)
    ahl01r061 = datadef.Node('AHL.0%s.R061' % floor_id)
    ahl01r062 = datadef.Node('AHL.0%s.R062' % floor_id)
    ahl01r063 = datadef.Node('AHL.0%s.R063' % floor_id)
    ahl01r064 = datadef.Node('AHL.0%s.R064' % floor_id)
    ahl01r065 = datadef.Node('AHL.0%s.R065' % floor_id)
    ahl01r066 = datadef.Node('AHL.0%s.R066' % floor_id)
    ahl01r067 = datadef.Node('AHL.0%s.R067' % floor_id)
    ahl01r068 = datadef.Node('AHL.0%s.R068' % floor_id)
    ahl01r069 = datadef.Node('AHL.0%s.R069' % floor_id)
    ahl01r070 = datadef.Node('AHL.0%s.R070' % floor_id)
    ahl01r071 = datadef.Node('AHL.0%s.R071' % floor_id)
    ahl01r072 = datadef.Node('AHL.0%s.R072' % floor_id)
    ahl01r073 = datadef.Node('AHL.0%s.R073' % floor_id)
    ahl01r074 = datadef.Node('AHL.0%s.R074' % floor_id)
    ahl01r075 = datadef.Node('AHL.0%s.R075' % floor_id)
    ahl01r076 = datadef.Node('AHL.0%s.R076' % floor_id)
    ahl01r077 = datadef.Node('AHL.0%s.R077' % floor_id)
    ahl01r078 = datadef.Node('AHL.0%s.R078' % floor_id)
    ahl01r079 = datadef.Node('AHL.0%s.R079' % floor_id)
    ahl01r080 = datadef.Node('AHL.0%s.R080' % floor_id)
    ahl01r081 = datadef.Node('AHL.0%s.R081' % floor_id)
    ahl01r082 = datadef.Node('AHL.0%s.R082' % floor_id)
    ahl01r083 = datadef.Node('AHL.0%s.R083' % floor_id)
    ahl01r084 = datadef.Node('AHL.0%s.R084' % floor_id)
    ahl01r085 = datadef.Node('AHL.0%s.R085' % floor_id)
    ahl01r086 = datadef.Node('AHL.0%s.R086' % floor_id)
    ahl01r087 = datadef.Node('AHL.0%s.R087' % floor_id)
    ahl01r088 = datadef.Node('AHL.0%s.R088' % floor_id)
    ahl01r089 = datadef.Node('AHL.0%s.R089' % floor_id)
    ahl01r090 = datadef.Node('AHL.0%s.R090' % floor_id)
    ahl01r091 = datadef.Node('AHL.0%s.R091' % floor_id)
    ahl01r092 = datadef.Node('AHL.0%s.R092' % floor_id)
    ahl01r093 = datadef.Node('AHL.0%s.R093' % floor_id)
    ahl01r094 = datadef.Node('AHL.0%s.R094' % floor_id)
    ahl01r095 = datadef.Node('AHL.0%s.R095' % floor_id)
    ahl01r096 = datadef.Node('AHL.0%s.R096' % floor_id)
    ahl01r097 = datadef.Node('AHL.0%s.R097' % floor_id)
    ahl01r098 = datadef.Node('AHL.0%s.R098' % floor_id)
    ahl01r099 = datadef.Node('AHL.0%s.R099' % floor_id)
    ahl01r100 = datadef.Node('AHL.0%s.R100' % floor_id)
    ahl01r101 = datadef.Node('AHL.0%s.R101' % floor_id)
    ahl01r102 = datadef.Node('AHL.0%s.R102' % floor_id)
    ahl01r103 = datadef.Node('AHL.0%s.R103' % floor_id)
    ahl01r104 = datadef.Node('AHL.0%s.R104' % floor_id)
    ahl01r105 = datadef.Node('AHL.0%s.R105' % floor_id)
    ahl01r106 = datadef.Node('AHL.0%s.R106' % floor_id)
    ahl01r107 = datadef.Node('AHL.0%s.R107' % floor_id)
    ahl01r108 = datadef.Node('AHL.0%s.R108' % floor_id)
    ahl01r109 = datadef.Node('AHL.0%s.R109' % floor_id)
    ahl01r110 = datadef.Node('AHL.0%s.R110' % floor_id)
    ahl01r111 = datadef.Node('AHL.0%s.R111' % floor_id)
    ahl01r112 = datadef.Node('AHL.0%s.R112' % floor_id)
    ahl01r113 = datadef.Node('AHL.0%s.R113' % floor_id)
    ahl01r114 = datadef.Node('AHL.0%s.R114' % floor_id)
    ahl01r115 = datadef.Node('AHL.0%s.R115' % floor_id)
    ahl01r116 = datadef.Node('AHL.0%s.R116' % floor_id)
    ahl01r117 = datadef.Node('AHL.0%s.R117' % floor_id)
    ahl01r118 = datadef.Node('AHL.0%s.R118' % floor_id)
    ahl01r119 = datadef.Node('AHL.0%s.R119' % floor_id)
    ahl01r120 = datadef.Node('AHL.0%s.R120' % floor_id)
    ahl01r121 = datadef.Node('AHL.0%s.R121' % floor_id)
    ahl01r122 = datadef.Node('AHL.0%s.R122' % floor_id)
    ahl01r123 = datadef.Node('AHL.0%s.R123' % floor_id)
    ahl01r124 = datadef.Node('AHL.0%s.R124' % floor_id)
    ahl01r125 = datadef.Node('AHL.0%s.R125' % floor_id)
    ahl01r126 = datadef.Node('AHL.0%s.R126' % floor_id)
    ahl01r127 = datadef.Node('AHL.0%s.R127' % floor_id)
    ahl01r128 = datadef.Node('AHL.0%s.R128' % floor_id)
    ahl01r129 = datadef.Node('AHL.0%s.R129' % floor_id)
    ahl01r130 = datadef.Node('AHL.0%s.R130' % floor_id)
    ahl01r131 = datadef.Node('AHL.0%s.R131' % floor_id)
    ahl01r132 = datadef.Node('AHL.0%s.R132' % floor_id)
    ahl01r133 = datadef.Node('AHL.0%s.R133' % floor_id)
    ahl01r134 = datadef.Node('AHL.0%s.R134' % floor_id)
    ahl01r135 = datadef.Node('AHL.0%s.R135' % floor_id)
    ahl01r136 = datadef.Node('AHL.0%s.R136' % floor_id)
    ahl01r137 = datadef.Node('AHL.0%s.R137' % floor_id)
    ahl01r138 = datadef.Node('AHL.0%s.R138' % floor_id)
    ahl01r139 = datadef.Node('AHL.0%s.R139' % floor_id)
    ahl01r140 = datadef.Node('AHL.0%s.R140' % floor_id)
    ahl01r141 = datadef.Node('AHL.0%s.R141' % floor_id)
    ahl01r142 = datadef.Node('AHL.0%s.R142' % floor_id)
    ahl01r143 = datadef.Node('AHL.0%s.R143' % floor_id)
    ahl01r144 = datadef.Node('AHL.0%s.R144' % floor_id)
    ahl01r145 = datadef.Node('AHL.0%s.R145' % floor_id)
    ahl01r146 = datadef.Node('AHL.0%s.R146' % floor_id)
    ahl01r147 = datadef.Node('AHL.0%s.R147' % floor_id)
    ahl01r148 = datadef.Node('AHL.0%s.R148' % floor_id)
    ahl01r149 = datadef.Node('AHL.0%s.R149' % floor_id)
    ahl01r150 = datadef.Node('AHL.0%s.R150' % floor_id)
    ahl01r151 = datadef.Node('AHL.0%s.R151' % floor_id)

    #
    # Add nodes to the graph
    #
    graph.add_node(ahl01s001)  # stairs
    graph.add_node(ahl01s002)
    graph.add_node(ahl01s003)
    graph.add_node(ahl01s004)
    graph.add_node(ahl01s005)
    graph.add_node(ahl01s006)
    graph.add_node(ahl01s007)

    graph.add_node(ahl01e001)  # elevators
    graph.add_node(ahl01e002)
    graph.add_node(ahl01e003)
    graph.add_node(ahl01e004)
    graph.add_node(ahl01e005)
    graph.add_node(ahl01e006)
    graph.add_node(ahl01e007)
    graph.add_node(ahl01e008)

    graph.add_node(ahl01c001)  # corridors
    graph.add_node(ahl01c002)
    graph.add_node(ahl01c003)
    graph.add_node(ahl01c004)
    graph.add_node(ahl01c005)
    graph.add_node(ahl01c006)
    graph.add_node(ahl01c007)
    graph.add_node(ahl01c008)
    graph.add_node(ahl01c009)
    graph.add_node(ahl01c010)
    graph.add_node(ahl01c011)
    graph.add_node(ahl01c012)
    graph.add_node(ahl01c013)
    graph.add_node(ahl01c014)
    graph.add_node(ahl01c015)
    graph.add_node(ahl01c016)
    graph.add_node(ahl01c017)
    graph.add_node(ahl01c018)
    graph.add_node(ahl01c019)
    graph.add_node(ahl01c020)
    graph.add_node(ahl01c021)
    graph.add_node(ahl01c022)
    graph.add_node(ahl01c023)
    graph.add_node(ahl01c024)
    graph.add_node(ahl01c025)
    graph.add_node(ahl01c026)
    graph.add_node(ahl01c027)
    graph.add_node(ahl01c028)
    graph.add_node(ahl01c029)
    graph.add_node(ahl01c030)
    graph.add_node(ahl01c031)
    graph.add_node(ahl01c032)
    graph.add_node(ahl01c033)
    graph.add_node(ahl01c034)
    graph.add_node(ahl01c035)
    graph.add_node(ahl01c036)
    graph.add_node(ahl01c037)

    graph.add_node(ahl01r001)  # rooms
    graph.add_node(ahl01r002)
    graph.add_node(ahl01r003)
    graph.add_node(ahl01r004)
    graph.add_node(ahl01r005)
    graph.add_node(ahl01r006)
    graph.add_node(ahl01r007)
    graph.add_node(ahl01r008)
    graph.add_node(ahl01r009)
    graph.add_node(ahl01r010)
    graph.add_node(ahl01r011)
    graph.add_node(ahl01r012)
    graph.add_node(ahl01r013)
    graph.add_node(ahl01r014)
    graph.add_node(ahl01r015)
    graph.add_node(ahl01r016)
    graph.add_node(ahl01r017)
    graph.add_node(ahl01r018)
    graph.add_node(ahl01r019)
    graph.add_node(ahl01r020)
    graph.add_node(ahl01r021)
    graph.add_node(ahl01r022)
    graph.add_node(ahl01r023)
    graph.add_node(ahl01r024)
    graph.add_node(ahl01r025)
    graph.add_node(ahl01r026)
    graph.add_node(ahl01r027)
    graph.add_node(ahl01r028)
    graph.add_node(ahl01r029)
    graph.add_node(ahl01r030)
    graph.add_node(ahl01r031)
    graph.add_node(ahl01r032)
    graph.add_node(ahl01r033)
    graph.add_node(ahl01r034)
    graph.add_node(ahl01r035)
    graph.add_node(ahl01r036)
    graph.add_node(ahl01r037)
    graph.add_node(ahl01r038)
    graph.add_node(ahl01r039)
    graph.add_node(ahl01r040)
    graph.add_node(ahl01r041)
    graph.add_node(ahl01r042)
    graph.add_node(ahl01r043)
    graph.add_node(ahl01r044)
    graph.add_node(ahl01r045)
    graph.add_node(ahl01r046)
    graph.add_node(ahl01r047)
    graph.add_node(ahl01r048)
    graph.add_node(ahl01r049)
    graph.add_node(ahl01r050)
    graph.add_node(ahl01r051)
    graph.add_node(ahl01r052)
    graph.add_node(ahl01r053)
    graph.add_node(ahl01r054)
    graph.add_node(ahl01r055)
    graph.add_node(ahl01r056)
    graph.add_node(ahl01r057)
    graph.add_node(ahl01r058)
    graph.add_node(ahl01r059)
    graph.add_node(ahl01r060)
    graph.add_node(ahl01r061)
    graph.add_node(ahl01r062)
    graph.add_node(ahl01r063)
    graph.add_node(ahl01r064)
    graph.add_node(ahl01r065)
    graph.add_node(ahl01r066)
    graph.add_node(ahl01r067)
    graph.add_node(ahl01r068)
    graph.add_node(ahl01r069)
    graph.add_node(ahl01r070)
    graph.add_node(ahl01r071)
    graph.add_node(ahl01r072)
    graph.add_node(ahl01r073)
    graph.add_node(ahl01r074)
    graph.add_node(ahl01r075)
    graph.add_node(ahl01r076)
    graph.add_node(ahl01r077)
    graph.add_node(ahl01r078)
    graph.add_node(ahl01r079)
    graph.add_node(ahl01r080)
    graph.add_node(ahl01r081)
    graph.add_node(ahl01r082)
    graph.add_node(ahl01r083)
    graph.add_node(ahl01r084)
    graph.add_node(ahl01r085)
    graph.add_node(ahl01r086)
    graph.add_node(ahl01r087)
    graph.add_node(ahl01r088)
    graph.add_node(ahl01r089)
    graph.add_node(ahl01r090)
    graph.add_node(ahl01r091)
    graph.add_node(ahl01r092)
    graph.add_node(ahl01r093)
    graph.add_node(ahl01r094)
    graph.add_node(ahl01r095)
    graph.add_node(ahl01r096)
    graph.add_node(ahl01r097)
    graph.add_node(ahl01r098)
    graph.add_node(ahl01r099)
    graph.add_node(ahl01r100)
    graph.add_node(ahl01r101)
    graph.add_node(ahl01r102)
    graph.add_node(ahl01r103)
    graph.add_node(ahl01r104)
    graph.add_node(ahl01r105)
    graph.add_node(ahl01r106)
    graph.add_node(ahl01r107)
    graph.add_node(ahl01r108)
    graph.add_node(ahl01r109)
    graph.add_node(ahl01r110)
    graph.add_node(ahl01r111)
    graph.add_node(ahl01r112)
    graph.add_node(ahl01r113)
    graph.add_node(ahl01r114)
    graph.add_node(ahl01r115)
    graph.add_node(ahl01r116)
    graph.add_node(ahl01r117)
    graph.add_node(ahl01r118)
    graph.add_node(ahl01r119)
    graph.add_node(ahl01r120)
    graph.add_node(ahl01r121)
    graph.add_node(ahl01r122)
    graph.add_node(ahl01r123)
    graph.add_node(ahl01r124)
    graph.add_node(ahl01r125)
    graph.add_node(ahl01r126)
    graph.add_node(ahl01r127)
    graph.add_node(ahl01r128)
    graph.add_node(ahl01r129)
    graph.add_node(ahl01r130)
    graph.add_node(ahl01r131)
    graph.add_node(ahl01r132)
    graph.add_node(ahl01r133)
    graph.add_node(ahl01r134)
    graph.add_node(ahl01r135)
    graph.add_node(ahl01r136)
    graph.add_node(ahl01r137)
    graph.add_node(ahl01r138)
    graph.add_node(ahl01r139)
    graph.add_node(ahl01r140)
    graph.add_node(ahl01r141)
    graph.add_node(ahl01r142)
    graph.add_node(ahl01r143)
    graph.add_node(ahl01r144)
    graph.add_node(ahl01r145)
    graph.add_node(ahl01r146)
    graph.add_node(ahl01r147)
    graph.add_node(ahl01r148)
    graph.add_node(ahl01r149)
    graph.add_node(ahl01r150)
    graph.add_node(ahl01r151)

    #
    # Connect nodes via edges
    #
    # Almost every room/elevator/staircase connect to a corridor, thus to make
    # things a bit easier the corridors are on the left hand side:
    # e.g. graph.add_edge(<corridor>, <r/e/s>)
    graph.add_edge(ahl01c001, ahl01s001)
    graph.add_edge(ahl01c001, ahl01c002)
    graph.add_edge(ahl01c002, ahl01c003)
    graph.add_edge(ahl01c002, ahl01c004)
    graph.add_edge(ahl01c003, ahl01c004)
    graph.add_edge(ahl01c004, ahl01c005)
    graph.add_edge(ahl01c004, ahl01c006)
    graph.add_edge(ahl01c004, ahl01r001)
    graph.add_edge(ahl01c004, ahl01r002)
    graph.add_edge(ahl01c004, ahl01r003)
    graph.add_edge(ahl01c004, ahl01r004)
    graph.add_edge(ahl01c004, ahl01r005)
    graph.add_edge(ahl01c004, ahl01r006)
    graph.add_edge(ahl01c004, ahl01r007)
    graph.add_edge(ahl01c004, ahl01r015)
    graph.add_edge(ahl01c004, ahl01r016)
    graph.add_edge(ahl01c004, ahl01r019)
    graph.add_edge(ahl01c005, ahl01c011)
    graph.add_edge(ahl01c005, ahl01r024)
    graph.add_edge(ahl01c005, ahl01r025)
    graph.add_edge(ahl01c006, ahl01c010)
    graph.add_edge(ahl01c006, ahl01c007)
    graph.add_edge(ahl01c006, ahl01r008)
    graph.add_edge(ahl01c006, ahl01r009)
    graph.add_edge(ahl01c006, ahl01r010)
    graph.add_edge(ahl01c006, ahl01r011)
    graph.add_edge(ahl01c006, ahl01r012)
    graph.add_edge(ahl01c006, ahl01r027)
    graph.add_edge(ahl01c007, ahl01c008)
    graph.add_edge(ahl01c007, ahl01e001)
    graph.add_edge(ahl01c007, ahl01e002)
    graph.add_edge(ahl01c007, ahl01r013)
    graph.add_edge(ahl01c007, ahl01r034)
    graph.add_edge(ahl01c007, ahl01r035)
    graph.add_edge(ahl01c007, ahl01r036)
    graph.add_edge(ahl01c008, ahl01s002)
    graph.add_edge(ahl01c008, ahl01c014)
    graph.add_edge(ahl01c008, ahl01r014)
    graph.add_edge(ahl01c008, ahl01r037)
    graph.add_edge(ahl01c009, ahl01c011)
    graph.add_edge(ahl01c009, ahl01r016)
    graph.add_edge(ahl01c009, ahl01r017)
    graph.add_edge(ahl01c009, ahl01r018)
    graph.add_edge(ahl01c009, ahl01r020)
    graph.add_edge(ahl01c009, ahl01r021)
    graph.add_edge(ahl01c009, ahl01r022)
    graph.add_edge(ahl01c010, ahl01c011)
    graph.add_edge(ahl01c010, ahl01r026)
    graph.add_edge(ahl01c010, ahl01r028)
    graph.add_edge(ahl01c010, ahl01r029)
    graph.add_edge(ahl01c010, ahl01r031)
    graph.add_edge(ahl01c010, ahl01r032)
    graph.add_edge(ahl01c011, ahl01c012)
    graph.add_edge(ahl01c011, ahl01r023)
    graph.add_edge(ahl01c011, ahl01r030)
    graph.add_edge(ahl01c011, ahl01r033)
    graph.add_edge(ahl01c011, ahl01r034)
    graph.add_edge(ahl01c012, ahl01c013)
    graph.add_edge(ahl01c012, ahl01c016)
    graph.add_edge(ahl01c012, ahl01c018)
    graph.add_edge(ahl01c012, ahl01r035)
    graph.add_edge(ahl01c012, ahl01r042)
    graph.add_edge(ahl01c012, ahl01r044)
    graph.add_edge(ahl01c012, ahl01r045)
    graph.add_edge(ahl01c012, ahl01r050)
    graph.add_edge(ahl01c012, ahl01r051)
    graph.add_edge(ahl01c013, ahl01c014)
    graph.add_edge(ahl01c013, ahl01r043)
    graph.add_edge(ahl01c014, ahl01c015)
    graph.add_edge(ahl01c014, ahl01c016)
    graph.add_edge(ahl01c014, ahl01r038)
    graph.add_edge(ahl01c014, ahl01r051)
    graph.add_edge(ahl01c014, ahl01r052)
    graph.add_edge(ahl01c014, ahl01r053)
    graph.add_edge(ahl01c014, ahl01r047)
    graph.add_edge(ahl01c014, ahl01r048)
    graph.add_edge(ahl01c014, ahl01r056)
    graph.add_edge(ahl01c014, ahl01r055)
    graph.add_edge(ahl01c015, ahl01r039)
    graph.add_edge(ahl01c015, ahl01r040)
    graph.add_edge(ahl01c015, ahl01r041)
    graph.add_edge(ahl01c015, ahl01r049)
    graph.add_edge(ahl01c016, ahl01s003)
    graph.add_edge(ahl01c016, ahl01c017)
    graph.add_edge(ahl01c016, ahl01r054)
    graph.add_edge(ahl01c017, ahl01c019)
    graph.add_edge(ahl01c017, ahl01c029)
    graph.add_edge(ahl01c017, ahl01e003)
    graph.add_edge(ahl01c017, ahl01e004)
    graph.add_edge(ahl01c017, ahl01r096)
    graph.add_edge(ahl01c017, ahl01r101)
    graph.add_edge(ahl01c017, ahl01r103)
    graph.add_edge(ahl01c018, ahl01c019)
    graph.add_edge(ahl01c018, ahl01c021)
    graph.add_edge(ahl01c019, ahl01c020)
    graph.add_edge(ahl01c019, ahl01c029)
    graph.add_edge(ahl01c019, ahl01c028)
    graph.add_edge(ahl01c019, ahl01r084)
    graph.add_edge(ahl01c020, ahl01c023)
    graph.add_edge(ahl01c020, ahl01r083)
    graph.add_edge(ahl01c020, ahl01r082)
    graph.add_edge(ahl01c020, ahl01r081)
    graph.add_edge(ahl01c021, ahl01c022)
    graph.add_edge(ahl01c021, ahl01c023)
    graph.add_edge(ahl01c021, ahl01r065)
    graph.add_edge(ahl01c021, ahl01r064)
    graph.add_edge(ahl01c021, ahl01r063)
    graph.add_edge(ahl01c021, ahl01r062)
    graph.add_edge(ahl01c021, ahl01r080)
    graph.add_edge(ahl01c022, ahl01c025)
    graph.add_edge(ahl01c022, ahl01r061)
    graph.add_edge(ahl01c022, ahl01r060)
    graph.add_edge(ahl01c022, ahl01r059)
    graph.add_edge(ahl01c022, ahl01r057)
    graph.add_edge(ahl01c022, ahl01r073)
    graph.add_edge(ahl01c022, ahl01r074)
    graph.add_edge(ahl01c022, ahl01r075)
    graph.add_edge(ahl01c022, ahl01r076)
    graph.add_edge(ahl01c023, ahl01c024)
    graph.add_edge(ahl01c023, ahl01r078)
    graph.add_edge(ahl01c023, ahl01r079)
    graph.add_edge(ahl01c023, ahl01r095)
    graph.add_edge(ahl01c023, ahl01r094)
    graph.add_edge(ahl01c024, ahl01c025)
    graph.add_edge(ahl01c024, ahl01c026)
    graph.add_edge(ahl01c024, ahl01r077)
    graph.add_edge(ahl01c024, ahl01r075)
    graph.add_edge(ahl01c024, ahl01r074)
    graph.add_edge(ahl01c024, ahl01r073)
    graph.add_edge(ahl01c024, ahl01r070)
    graph.add_edge(ahl01c024, ahl01r068)
    graph.add_edge(ahl01c024, ahl01r093)
    graph.add_edge(ahl01c024, ahl01r092)
    graph.add_edge(ahl01c024, ahl01r091)
    graph.add_edge(ahl01c024, ahl01r090)
    graph.add_edge(ahl01c024, ahl01r089)
    graph.add_edge(ahl01c024, ahl01r087)
    graph.add_edge(ahl01c024, ahl01r086)
    graph.add_edge(ahl01c025, ahl01r071)
    graph.add_edge(ahl01c025, ahl01r072)
    graph.add_edge(ahl01c026, ahl01s004)
    graph.add_edge(ahl01c026, ahl01c027)
    graph.add_edge(ahl01c026, ahl01r085)
    graph.add_edge(ahl01c027, ahl01s004)
    graph.add_edge(ahl01c028, ahl01r097)
    graph.add_edge(ahl01c028, ahl01r098)
    graph.add_edge(ahl01c028, ahl01r099)
    graph.add_edge(ahl01c028, ahl01r100)
    graph.add_edge(ahl01c028, ahl01r102)
    graph.add_edge(ahl01c028, ahl01r103)
    graph.add_edge(ahl01c029, ahl01c030)
    graph.add_edge(ahl01c029, ahl01r104)
    graph.add_edge(ahl01c030, ahl01s005)
    graph.add_edge(ahl01c030, ahl01c031)
    graph.add_edge(ahl01c030, ahl01e005)
    graph.add_edge(ahl01c030, ahl01e006)
    graph.add_edge(ahl01c030, ahl01r105)
    graph.add_edge(ahl01c031, ahl01c032)
    graph.add_edge(ahl01c031, ahl01c033)
    graph.add_edge(ahl01c031, ahl01r124)
    graph.add_edge(ahl01c031, ahl01r122)
    graph.add_edge(ahl01c031, ahl01r123)
    graph.add_edge(ahl01c031, ahl01r108)
    graph.add_edge(ahl01c031, ahl01r107)
    graph.add_edge(ahl01c031, ahl01r106)
    graph.add_edge(ahl01c032, ahl01s006)
    graph.add_edge(ahl01c032, ahl01e007)
    graph.add_edge(ahl01c032, ahl01e008)
    graph.add_edge(ahl01c032, ahl01c035)
    graph.add_edge(ahl01c032, ahl01r128)
    graph.add_edge(ahl01c032, ahl01r130)
    graph.add_edge(ahl01c032, ahl01r131)
    graph.add_edge(ahl01c033, ahl01c034)
    graph.add_edge(ahl01c033, ahl01c035)
    graph.add_edge(ahl01c033, ahl01r125)
    graph.add_edge(ahl01c033, ahl01r126)
    graph.add_edge(ahl01c033, ahl01r127)
    graph.add_edge(ahl01c033, ahl01r129)
    graph.add_edge(ahl01c033, ahl01r132)
    graph.add_edge(ahl01c033, ahl01r138)
    graph.add_edge(ahl01c033, ahl01r139)
    graph.add_edge(ahl01c033, ahl01r140)
    graph.add_edge(ahl01c033, ahl01r141)
    graph.add_edge(ahl01c033, ahl01r109)
    graph.add_edge(ahl01c033, ahl01r110)
    graph.add_edge(ahl01c033, ahl01r111)
    graph.add_edge(ahl01c033, ahl01r112)
    graph.add_edge(ahl01c033, ahl01r115)
    graph.add_edge(ahl01c033, ahl01r116)
    graph.add_edge(ahl01c033, ahl01r117)
    graph.add_edge(ahl01c033, ahl01r118)
    graph.add_edge(ahl01c033, ahl01r119)
    graph.add_edge(ahl01c033, ahl01r120)
    graph.add_edge(ahl01c033, ahl01r121)
    graph.add_edge(ahl01c034, ahl01c035)
    graph.add_edge(ahl01c034, ahl01r137)
    graph.add_edge(ahl01c034, ahl01r136)
    graph.add_edge(ahl01c034, ahl01r135)
    graph.add_edge(ahl01c035, ahl01c036)
    graph.add_edge(ahl01c035, ahl01r134)
    graph.add_edge(ahl01c035, ahl01r143)
    graph.add_edge(ahl01c035, ahl01r144)
    graph.add_edge(ahl01c035, ahl01r145)
    graph.add_edge(ahl01c035, ahl01r146)
    graph.add_edge(ahl01c035, ahl01r147)
    graph.add_edge(ahl01c035, ahl01r148)
    graph.add_edge(ahl01c035, ahl01r149)
    graph.add_edge(ahl01c035, ahl01r150)
    graph.add_edge(ahl01c035, ahl01r138)
    graph.add_edge(ahl01c035, ahl01r139)
    graph.add_edge(ahl01c036, ahl01s007)
    graph.add_edge(ahl01c036, ahl01c037)
    graph.add_edge(ahl01c036, ahl01r142)
    graph.add_edge(ahl01c037, ahl01s007)

    # Special cases
    graph.add_edge(ahl01r016, ahl01r020)
    graph.add_edge(ahl01r038, ahl01r040)
    graph.add_edge(ahl01r038, ahl01r041)
    graph.add_edge(ahl01r050, ahl01r046)
    graph.add_edge(ahl01r060, ahl01r059)
    graph.add_edge(ahl01r060, ahl01r061)
    graph.add_edge(ahl01r057, ahl01r058)
    graph.add_edge(ahl01r068, ahl01r069)
    graph.add_edge(ahl01r068, ahl01r066)
    graph.add_edge(ahl01r068, ahl01r067)
    graph.add_edge(ahl01r088, ahl01r087)
    graph.add_edge(ahl01r113, ahl01r112)
    graph.add_edge(ahl01r113, ahl01r115)
    graph.add_edge(ahl01r114, ahl01r112)
    graph.add_edge(ahl01r114, ahl01r115)
    graph.add_edge(ahl01r134, ahl01r133)
    graph.add_edge(ahl01r150, ahl01r151)

    #
    # Stairs and elevators are regarded as connectors
    #
    connectors = [ahl01s001, ahl01s002, ahl01s003, ahl01s004, ahl01s005,
                  ahl01s006, ahl01s007, ahl01e001, ahl01e002, ahl01e003,
                  ahl01e004, ahl01e005, ahl01e006, ahl01e007, ahl01e008]

    return (graph, connectors)
